@@ -6,9 +6,9 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-use crate::messages::{Message, MessageHash};
+use crate::messages::RoutingMsgUtils;
 use lru_time_cache::LruCache;
-use sn_messaging::DstLocation;
+use sn_messaging::{node::RoutingMsg, DstLocation, MessageId};
 use std::time::Duration;
 use xor_name::XorName;
 
@@ -36,8 +36,8 @@ impl FilteringResult {
 
 // Structure to filter (throttle) incoming and outgoing messages.
 pub(crate) struct MessageFilter {
-    incoming: LruCache<MessageHash, ()>,
-    outgoing: LruCache<(MessageHash, XorName), ()>,
+    incoming: LruCache<MessageId, ()>,
+    outgoing: LruCache<(MessageId, XorName), ()>,
 }
 
 impl MessageFilter {
@@ -54,35 +54,34 @@ impl MessageFilter {
         }
     }
 
-    pub fn contains_incoming(&self, msg: &Message) -> bool {
-        let hash = msg.hash();
-        self.incoming.contains_key(hash)
-    }
-
-    pub fn insert_incoming(&mut self, msg: &Message) {
-        // Not filtering direct messages.
-        if let DstLocation::DirectAndUnrouted = msg.dst() {
-            return;
-        }
-        let _ = self.incoming.insert(*msg.hash(), ());
-    }
-
     // Filter outgoing `SNRoutingMessage`. Return whether this specific message has been seen recently
     // (and thus should not be sent, due to deduplication).
     //
-    pub fn filter_outgoing(&mut self, msg: &Message, pub_id: &XorName) -> FilteringResult {
+    pub fn filter_outgoing(&mut self, msg: &RoutingMsg, pub_id: &XorName) -> FilteringResult {
         // Not filtering direct messages.
         if let DstLocation::DirectAndUnrouted = msg.dst() {
             return FilteringResult::NewMessage;
         }
 
-        if self.outgoing.insert((*msg.hash(), *pub_id), ()).is_some() {
+        if self.outgoing.insert((msg.id(), *pub_id), ()).is_some() {
+            debug!("&&&& Outgoing message filtered: {:?}", msg);
             FilteringResult::KnownMessage
         } else {
             FilteringResult::NewMessage
         }
     }
 
+    // Returns `true` if not already having it.
+    pub fn add_to_filter(&mut self, msg_id: &MessageId) -> bool {
+        let cur_value = self.incoming.insert(*msg_id, ());
+
+        if cur_value.is_some() {
+            debug!("&&&& Incoming message filtered: {:?}", msg_id);
+        }
+
+        cur_value.is_none()
+    }
+
     // Resets both incoming and outgoing filters.
     pub fn reset(&mut self) {
         self.incoming.clear();
