@@ -7,8 +7,8 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use crate::event::Event;
+use std::fmt::{self, Debug, Formatter};
 use tokio::sync::mpsc;
-
 /// Stream of routing node events
 pub struct EventStream {
     events_rx: mpsc::UnboundedReceiver<Event>,
@@ -24,3 +24,9 @@ impl EventStream {
         self.events_rx.recv().await
     }
 }
+
+impl Debug for EventStream {
+    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+        write!(f, "EventStream")
+    }
+}
